#!/usr/bin/python

from opyscad import *
import math

tube_height = 20
tube_thickness = 3
tube_in_radius = 54/2 + 0.3
tube_in_radius2 = 25.9
tube_out_radius = tube_in_radius + tube_thickness
tube_notch_size = 1.5
brim_height = 4

red_mark_radius = 1
red_mark_offset = 2
red_mark_depth = 1

ledge_height = 2.5
ledge_offset = 4.4
ledge_angles = [[346.5, 47],
				[113.5, 174.5],
				[233, 292.3]]

stopper_height = 5
stopper_width = 2
bayounet1_latch_radius = 0.3
bayounet1_latch_height = 2

angle_delta = 45

b2_out_radius = 54/2 - 0.2
b2_out_radius2 = 25.5
b2_thickness = 2

ledge2_height = 2
ledge2_offset = 5.6
ledge2_angles = [[53, 112],
				[180.5, 230.5],
				[296, 344.5]]
b2_brim_height = 1.7

main_fn = 100
notch_fn = 10
red_mark_fn = 15



def tube(in_radius, out_radius, height, notch_size):
	res = cylinder(height, out_radius, fn=main_fn)
	
	res -= cylinder(height + 2, in_radius, fn=main_fn) << [0, 0, -1]
	
	if notch_size > 0:
		notch = cylinder(height - brim_height*2, float(notch_size)/2, fn=notch_fn) << [out_radius, 0, brim_height]
		
		out_len = out_radius * math.pi * 2
		
		notch_cnt = int(out_len/notch_size/2)
		
		angle = 360.0/notch_cnt
		
		for i in range(notch_cnt):
			notch /= [0, 0, angle]
			res -= notch
			
	red_mark = cylinder(red_mark_depth + 1, red_mark_radius, fn=red_mark_fn)
	red_mark /= [0, 90, 0]
	res -= red_mark<< [-out_radius - 1, 0, red_mark_offset]
	red_mark <<= [-out_radius - 1, 0, height - red_mark_offset]
	red_mark /= [0, 0, angle_delta]
	res -= red_mark
			
	return res

def beveled_ring(r1, r2, r3, height):
	p = polygon([[r1, 0],
				[r2, 0],
				[r3, abs(r2 - r3)],
				[r3, height],
				[r1, height]])
	p /= [90,0,0]
	
	return rotate_extrude(fn=main_fn)(p)

def segment(obj, cutter, begin, end):
	res = obj - (cutter / [0, 0, begin])
	res -= (cutter / [0, 0, end - 180])
	return res

def bayonet1_latch(angle):
	latch = cylinder(bayounet1_latch_height, bayounet1_latch_radius)
	latch <<= [-tube_in_radius, 0, ledge_offset]
	
	latch /= [0, 0, angle]
	return latch
	
def bayonet1():
	ring = beveled_ring(tube_in_radius + 0.1, tube_in_radius, tube_in_radius2, ledge_height)
	ring <<= [0, 0, ledge_offset - ledge_height]
	
	cutter = cube([tube_in_radius*2 + 2, tube_in_radius + 1, ledge_offset + 1])
	cutter <<= [-tube_in_radius - 1, 0, 0]
	
	stopper_angle = stopper_width/(tube_in_radius * math.pi * 2) * 360
	
	stopper_ring = beveled_ring(tube_in_radius + 0.1, tube_in_radius, tube_in_radius2, stopper_height)
	stopper_ring <<= [0, 0, ledge_offset - ledge_height]
	
	stopper_cutter = cube([tube_in_radius*2 + 2, tube_in_radius + 1, stopper_height + 2])
	stopper_cutter <<= [-tube_in_radius - 1, 0, ledge_height - 1]
	
	res = union()
	for beg, end in ledge_angles:
		res += segment(ring, cutter, beg, end)
		res += segment(stopper_ring, stopper_cutter, end - stopper_angle, end)
		
	#res += bayonet1_latch(ledge2_angles[0][0])
	#res += bayonet1_latch(ledge2_angles[1][0])
	
	return res

def bayonet2():
	res = cylinder(b2_brim_height + 0.1, b2_out_radius, fn=main_fn)
	res <<= [0, 0, -0.1]
	c = cylinder(ledge2_offset - b2_brim_height + 0.1, b2_out_radius2, fn=main_fn)
	res += c << [0, 0, b2_brim_height - 0.1]
	
	res -= cylinder(ledge2_offset + 2, b2_out_radius2 - b2_thickness, fn=main_fn) << [0, 0, -1]

	h = (tube_in_radius - (b2_out_radius2 - b2_thickness)) * 2
	b = beveled_ring(tube_in_radius + 0.1, tube_in_radius, b2_out_radius2 - b2_thickness, h)
	res += b << [0, 0, -h]

	ring = beveled_ring(b2_out_radius2 - 0.1, b2_out_radius2, b2_out_radius, ledge2_height)
	ring <<= [0, 0, ledge2_offset - ledge2_height]
	
	cutter = cube([b2_out_radius*2 + 2, b2_out_radius + 1, ledge2_offset + 1])
	cutter <<= [-b2_out_radius - 1, 0, 0]
	
	stopper_angle = stopper_width/(b2_out_radius * math.pi * 2) * 360

	stopper_ring = beveled_ring(b2_out_radius2 - 0.1, b2_out_radius2, b2_out_radius, stopper_height)
	stopper_ring = cylinder(stopper_height, b2_out_radius)
	stopper_ring -= cylinder(stopper_height + 2, b2_out_radius2 - 0.1) << [0, 0, -1]
	stopper_ring <<= [0, 0, ledge2_offset - stopper_height]
	
	stopper_cutter = cube([b2_out_radius*2 + 2, b2_out_radius + 1, stopper_height + 2])
	stopper_cutter <<= [-b2_out_radius - 1, 0, ledge2_height - 1]

	for beg, end in ledge2_angles:
		beg += angle_delta
		end += angle_delta
		res += segment(ring, cutter, beg, end)
		
	beg, end = ledge2_angles[0]
	beg += angle_delta
	res += segment(stopper_ring, stopper_cutter, beg - stopper_angle, beg + 0.1)
	
	res <<= [0, 0, tube_height]
	
	return res

if __name__ == '__main__':
	res = tube(tube_in_radius, tube_out_radius, tube_height, tube_notch_size)
	
	res += bayonet1()
	res += bayonet2()
	
	res.save('tube.scad')



